/*
 * Testprogram for robotsimulator
 *
 * Bryterne har fgende funksjon:
 * bryter 0: Kjr forover
 * bryter 1: Kjr bakover
 * bryter 2: Sving hyre
 * bryter 3: Sving venstre
 * bryter 4: Snu radar 30 grader mot hyre, send radarpuls
 * bryter 5: Snu radar 30 grader mot venstre, send radarpuls
 * bryter 6: Send radarpuls
 *
 * Alle data som overfres fra simulatoren vises p displayet
 *
 */

#include "Robot_io.h"
#include <stk600/display.h>
#include <stk600/stk_io.h>
#include <stk600/timer.h>

unsigned int heading = 0;
position_type pos = {0,0};
unsigned int distance = 0;
int direction = 0;

void data_process()
{
	static boolean init = false;
	static Timer t1,t2;
	static enum {starter_data_henting, venter_heading_data, venter_pos_data}
		tilstand = starter_data_henting;

	if (!init)
	{
		init = true;
		t1 = newTimer();
		t2 = newTimer();
		startTimer(100,t1);
	}
	switch (tilstand)
	{
		case starter_data_henting:
			if (timeout(t1))
			{
				startTimer(100,t1);
				getHeading();
				startTimer(50,t2);
				tilstand = venter_heading_data;
			}
			break;
		case venter_heading_data:
			if (headingDataReady(&heading))
			{
				startTimer(50,t2);
				getPosition();
				tilstand = venter_pos_data;
			}
			else if (timeout(t2))
			{
				getPosition();
				tilstand = venter_pos_data;
			}
			break;
		case venter_pos_data:
			if (positionDataReady(&pos))
			{
				tilstand = starter_data_henting;
			}
			else if (timeout(t2))
			{
				tilstand = starter_data_henting;
			}
			break;
	}
}

void command_process()
{
	static Timer t1,t2;
	static boolean init = false;
	unsigned char buttons;
	static boolean b0was=false, b1was=false, b2was=false, b3was=false,
			       b4was=false, b5was=false, b6was=false;
    boolean is;
    static enum {tomgang, venter_radar_data} tilstand = tomgang;

	if (!init)
	{
		init = true;
		t1 = newTimer();
		t2 = newTimer();
		startTimer(500,t1);
		setSwing(0);
		setSpeed(0);
	}

	switch (tilstand)
	{
		case tomgang:
			if (timeout(t1))
			{
				startTimer(100,t1);

				buttons = switchRead();

				is = ((buttons & (1<<0)) != 0);
				if (is && !b0was)
					setSpeed(5);
				else if (!is && b0was)
					setSpeed(0);
				b0was = is;

				is = ((buttons & (1<<1)) != 0);
				if (is && !b1was)
					setSpeed(-5);
				else if (!is && b1was)
					setSpeed(0);
				b1was = is;

				is = ((buttons & (1<<2)) != 0);
				if (is && !b2was)
					setSwing(5);
				else if (!is && b2was)
					setSwing(0);
				b2was = is;

				is = ((buttons & (1<<3)) != 0);
				if (is && !b3was)
					setSwing(-5);
				else if (!is && b3was)
					setSwing(0);
				b3was = is;

				is = ((buttons & (1<<4)) != 0);
				if (is && !b4was)
				{
					direction += 30;
					if (direction >= 360)
						direction -= 360;
					radarPing(direction);
					startTimer(50,t2);
					tilstand = venter_radar_data;
				}
				b4was = is;

				is = ((buttons & (1<<5)) != 0);
				if (is && !b5was)
				{
					direction -= 30;
					if (direction < 0)
						direction += 360;
					radarPing(direction);
					startTimer(50,t2);
					tilstand = venter_radar_data;
				}
				b5was = is;

				is = ((buttons & (1<<6)) != 0);
				if (is && !b6was)
				{
					radarPing(direction);
					startTimer(50,t2);
					tilstand = venter_radar_data;
				}
				b6was = is;
			}
			break;
		case venter_radar_data:
			if (radarDataReady(&distance))
			{
				tilstand = tomgang;
			}
			else if (timeout(t2))
				tilstand = tomgang;
			break;
	}
}

void display_process()
{
	static Timer t;
	static boolean init = false;

	if (!init)
	{
		init = true;
		t = newTimer();
		startTimer(100,t);
		DSP_set_cursor(hidden);
		DSP_write('\f');
	}

	if (timeout(t))
	{
		startTimer(100,t);
		DSP_move_cursor(1,1);
		DSP_print("Heading: %3u  Position: x: %3d, y: %3d",heading, pos.x, pos.y);
		DSP_move_cursor(1,2);
		DSP_print("Retning: %3u  avstand: %3d cm",direction, distance);
		DSP_move_cursor (34,2);
		if (obstacleHit())
			DSP_write_string("CRASH");
		else
			DSP_write_string("     ");
	}
}

int main()
{
	while(1)
	{
		data_process();
		command_process();
		display_process();
	}
	return 0;
}
