/*
 * Testprogram for robotsimulator
 *
 * Bryterne har fgende funksjon:
 * bryter 0: Kjr forover
 * bryter 1: Kjr bakover
 * bryter 2: Sving hyre
 *
 * Posisjon og kompassretning vises p displayte.
 *
 * Radaren sender ut pulser i 5 forskjellige retninger,
 * og de mlte avstandene vises p displayet.
 *
 */

#include "Robot_io.h"
#include <stk600/display.h>
#include <stk600/stk_io.h>
#include <stk600/timer.h>

unsigned int heading = 0;
position_type pos = {0,0};
unsigned int d1,d2,d3,d4,d5;

void data_process()
{
	static boolean init = false;
	static Timer t1,t2;
	static enum {starter_data_henting, venter_heading_data, venter_pos_data}
		tilstand = starter_data_henting;

	if (!init)
	{
		init = true;
		t1 = newTimer();
		t2 = newTimer();
		startTimer(100,t1);
	}
	switch (tilstand)
	{
		case starter_data_henting:
			if (timeout(t1))
			{
				startTimer(100,t1);
				getHeading();
				startTimer(50,t2);
				tilstand = venter_heading_data;
			}
			break;
		case venter_heading_data:
			if (headingDataReady(&heading))
			{
				startTimer(50,t2);
				getPosition();
				tilstand = venter_pos_data;
			}
			else if (timeout(t2))
			{
				getPosition();
				tilstand = venter_pos_data;
			}
			break;
		case venter_pos_data:
			if (positionDataReady(&pos))
			{
				tilstand = starter_data_henting;
			}
			else if (timeout(t2))
			{
				tilstand = starter_data_henting;
			}
			break;
	}
}

void radar_process()
{
	static boolean init = false;
	static enum{starter_radar, venter270, venter315, venter0, venter45, venter90}
		state = starter_radar;
	static Timer t1,t2;

	if (!init)
	{
		init = true;
		t1 = newTimer();
		t2 = newTimer();
		startTimer(100,t1);
	}
	switch (state)
	{
		case starter_radar:
			if (timeout(t1))
			{
				startTimer(100,t1);
				radarPing(270);
				startTimer(50,t2);
				state = venter270;
			}
			break;
		case venter270:
			if (radarDataReady(&d1))
			{
				radarPing(315);
				startTimer(50,t2);
				state = venter315;
			}
			else if (timeout(t2))
			{
				radarPing(315);
				startTimer(50,t2);
				state = venter315;
			}
			break;
		case venter315:
			if (radarDataReady(&d2))
			{
				radarPing(0);
				startTimer(50,t2);
				state = venter0;
			}
			else if (timeout(t2))
			{
				radarPing(0);
				startTimer(50,t2);
				state = venter0;
			}
			break;
		case venter0:
			if (radarDataReady(&d3))
			{
				radarPing(45);
				startTimer(50,t2);
				state = venter45;
			}
			else if (timeout(t2))
			{
				radarPing(45);
				startTimer(50,t2);
				state = venter45;
			}
			break;
		case venter45:
			if (radarDataReady(&d4))
			{
				radarPing(90);
				startTimer(50,t2);
				state = venter90;
			}
			else if (timeout(t2))
			{
				radarPing(90);
				startTimer(50,t2);
				state = venter90;
			}
			break;
		case venter90:
			if (radarDataReady(&d5))
			{
				state = starter_radar;
			}
			else if (timeout(t2))
			{
				state = starter_radar;
			}
			break;
	}
}


void command_process()
{
	static Timer t1;
	static boolean init = false;
	unsigned char buttons;
	static boolean b0was=false, b1was=false, b2was=false, b3was=false;
    boolean is;

	if (!init)
	{
		init = true;
		t1 = newTimer();
		startTimer(500,t1);
		setSwing(0);
		setSpeed(0);
	}

	if (timeout(t1))
	{
		startTimer(100,t1);

		buttons = switchRead();

		is = ((buttons & (1<<0)) != 0);
		if (is && !b0was)
			setSpeed(5);
		else if (!is && b0was)
			setSpeed(0);
		b0was = is;

		is = ((buttons & (1<<1)) != 0);
		if (is && !b1was)
			setSpeed(-5);
		else if (!is && b1was)
			setSpeed(0);
		b1was = is;

		is = ((buttons & (1<<2)) != 0);
		if (is && !b2was)
			setSwing(5);
		else if (!is && b2was)
			setSwing(0);
		b2was = is;

		is = ((buttons & (1<<3)) != 0);
		if (is && !b3was)
			setSwing(-5);
		else if (!is && b3was)
			setSwing(0);
		b3was = is;
	}
}

void display_process()
{
	static Timer t;
	static boolean init = false;

	if (!init)
	{
		init = true;
		t = newTimer();
		startTimer(100,t);
		DSP_set_cursor(hidden);
		DSP_write('\f');
	}

	if (timeout(t))
	{
		startTimer(100,t);
		DSP_move_cursor(1,1);
		DSP_print("Heading: %3u  Position: x: %3d, y: %3d",heading, pos.x, pos.y);
		DSP_move_cursor(1,2);
		DSP_print("1:%4u 2:%4u 3:%4u 4:%4u 5:%4u", d1,d2,d3,d4,d5);
		DSP_move_cursor (36,2);
		if (obstacleHit())
			DSP_write_string("BOMP");
		else
			DSP_write_string("    ");
	}
}

int main()
{
	while(1)
	{
		data_process();
		radar_process();
		command_process();
		display_process();
	}
	return 0;
}
