
/*
* Lab06 Robot_Radar_Labyrint
*/
#include "Robot_io.h"
#include <stk600/stk_io.h>
#include <stk600/usart.h>
#include <stk600/timer.h>
#include <stk600/signals.h>
#include <stk600/display.h>
#include <avr/io.h>
#include <stdlib.h>
#include <stdio.h>

Signal radar0;
Signal radar1;
Signal radar2;
Signal pos;
Signal head;

Signal displayRadar0;
Signal displayRadar1;
Signal displayRadar2;
Signal displayPos;
Signal displayHead;
Signal displayDir;

void radar_process(void) 
{
	static int state = 0;
	static Timer t1, t2;
	static boolean init = false;
	static unsigned int radarDist0,radarDist1,radarDist2;
	
	if (!init) {
		init = true;
		t1 = newTimer();
		t2 = newTimer();
		startTimer(100,t1);
	}
	
	switch (state) {
		case 0:
			if (obstacleHit()) state = 4;
			if (timeout(t1)) {
				startTimer(100,t1);
				
				radarPing(270);							// start 270
				startTimer(25,t2);
				state = 1;
			}
			break;
		case 1:
			if (radarDataReady(&radarDist0)) {
				sendSignal(radar0, &radarDist0);
				sendSignal(displayRadar0, &radarDist0);	// end 270
				
				radarPing(90);							// start 90
				startTimer(25,t2);
				state = 2;
			} else if (timeout(t2)) {
				radarPing(90);
				startTimer(25,t2);
				state = 2;
			}
			break;
		case 2:
			if (radarDataReady(&radarDist2)) {
				sendSignal(radar2, &radarDist2);
				sendSignal(displayRadar2, &radarDist2);	// end 90
				
				radarPing(0);							// start 0
				startTimer(25,t2);
				state = 3;
			} else if (timeout(t2)) {
				radarPing(0);
				startTimer(25,t2);
				state = 3;
			}
			break;
		case 3:
			if (radarDataReady(&radarDist1)) {
				sendSignal(radar1, &radarDist1);
				sendSignal(displayRadar1, &radarDist1);	// end 0
				state = 0;
			} else if (timeout(t2)) {
				state = 0;
			}
			break;
		case 4:
			break;
	}
}

void data_process(void)
{
	static boolean init = false;
	static Timer t1,t2;
	position_type posData = {0,0};
    unsigned int headData = 0;
	static int state = 0;

	if (!init)
	{
		init = true;
		t1 = newTimer();
		t2 = newTimer();
		startTimer(100,t1);
	}

	switch (state) {
		case 0:
			if (obstacleHit()) state = 4;
			if (timeout(t1))
			{
				startTimer(100,t1);

				getPosition();					// pos
				startTimer(50,t2);
				state = 1;
			}
			break;
		case 1:
			if (positionDataReady(&posData)) {	
				sendSignal(pos, &posData);
				sendSignal(displayPos, &posData);
				getHeading();					// head
				startTimer(50,t2);
				state = 2;
			} else if (timeout(t2)) {
				getHeading();					// head
				startTimer(50,t2);
				state = 2;
			}
			break;
		case 2:
			if (headingDataReady(&headData)) {	
				sendSignal(head, &headData);
				sendSignal(displayHead, &headData);
				state = 0;
			} else if (timeout(t2)) {
				state = 0;
			}
			break;
		case 4:
			break;
	}
}

void display_process(void)
{
	static Timer t;
	static boolean init = false;
	static unsigned int radarDist0,radarDist1,radarDist2;
	static position_type posData = {0,0};
    static unsigned int headData = 0;
	static unsigned int stateData = 0;
	unsigned char dir = 'N';
	
	if (!init)
	{
		init = true;
		t = newTimer();
		startTimer(100,t);
		DSP_set_cursor(hidden);
		DSP_move_cursor(1,1);
		DSP_write('\f');
	}
	
	if (timeout(t))
	{
		startTimer(100,t);
		if (receivedSignal(displayRadar0,&radarDist0));
		emptySignalQueue(displayRadar0);
		if (receivedSignal(displayRadar1,&radarDist1));
		emptySignalQueue(displayRadar1);
		if (receivedSignal(displayRadar2,&radarDist2));
		emptySignalQueue(displayRadar2);
		if (receivedSignal(displayHead,&headData));
		emptySignalQueue(displayHead);
		if (receivedSignal(displayPos,&posData));
		emptySignalQueue(displayPos);
		if (receivedSignal(displayDir,&stateData));
		emptySignalQueue(displayDir);
		
		if (stateData == 0) 
			dir = 'N';
		else if (stateData == 1) 
			dir = 'S';
		else if (stateData == 2) 
			dir = 'E';
		else if (stateData == 3) 
			dir = 'W';
		
		DSP_move_cursor(1,1);
		DSP_print("Pos: x=%3d y=%3d  Head: %3d  Dir: %c", posData.x, posData.y, headData, dir);
		DSP_move_cursor(1,2);
		DSP_print("Dist:  0:%4u  1:%4u  2:%4u", radarDist0, radarDist1, radarDist2);
		DSP_move_cursor (36,2);
		if (obstacleHit()) {
			DSP_write_string("HIT!");
		} else {
			DSP_write_string("    ");
		}
	}
}


void control_process(void) 
{
	static Timer t3, t4;
	static boolean init = false;
	static unsigned	int radarDist0,radarDist1=1000,radarDist2=10;
	static position_type posData = {0,0};
    static unsigned int headData = 0;
	static unsigned int finalswing = 0;

	static enum {N,S,E,W,X,Y} dir = N;
	static enum {left,right,leftfast,rightfast,none} correction = none;
	static enum {turn,straight,navigate} swing = navigate;

	if (!init)
	 {
		init = true;
		t3 = newTimer();
		t4 = newTimer();
		startTimer(100,t3);
		startTimer(1,t4);
		setSwing(0);
		setSpeed(10);
	}

	if (timeout(t3))
	 {
		startTimer(100,t3);

		if (receivedSignal(radar0,&radarDist0));
		emptySignalQueue(radar0);
		if (receivedSignal(radar1,&radarDist1));
		emptySignalQueue(radar1);
		if (receivedSignal(radar2,&radarDist2));
		emptySignalQueue(radar2);
		if (receivedSignal(pos,&posData));
		emptySignalQueue(pos);
		if (receivedSignal(head,&headData));
		emptySignalQueue(head);

		// stopp ved ml
		if (dir == N && posData.x < 200 && posData.y >= 600)
		{
			dir = X;
		}

		// retning
		switch (dir) {
			// nord
			case N:
				 // sving timeout
				 switch (swing) {
				 		case turn:
							// stopp sving, kjr et stykke, g til straight modus
							if (timeout(t4)) {
							 	setSwing(0);

								// kjr frem litt
								swing = straight;
								startTimer(700,t4);
							}
							break;
				 		case straight:
							// stopp sving, sett hastighet, g til navigate modus
							if (timeout(t4)) {
							 	setSwing(0);
								setSpeed(10);
								swing = navigate;
							}
							break;
						case navigate:

							// pent til venstre, snu mot vest
							if (radarDist0 > 220) {
								setSwing(-2);
								swing = turn;
								startTimer(1150,t4);
								dir = W;

							// pent til hyre, snu mot st
							} else if (radarDist2 > 220 && radarDist1 < 180) {
								setSwing(2);
								swing = turn;
								startTimer(1150,t4);
								dir = E;

							// stengt forran, snu
							} else if (radarDist1 < 100) {
								setSwing(-10);
								setSpeed(0);
								swing = straight;
								startTimer(920,t4);
								dir = S;

							// rett frem, korrigering
							} else {
								if ((radarDist0 < 190 && radarDist2 < 190 && radarDist1 > 200 && // vegg begge sider
									radarDist0 < radarDist2-18 && // venstre lavere enn hyre
									((headData < 15) || (headData > 345)))) { // utenfor vinkel

									correction = rightfast;
								} else if ((radarDist0 < 190 && radarDist2 < 190 && radarDist1 > 200 && // vegg begge sider
									radarDist2 < radarDist0-18 &&  // hyre lavere enn venstre
									((headData > 345) || (headData < 15)))) {  // utenfor akseptabel vinkel

									correction = leftfast;
								} else if ((headData > 0+3 && headData < 180)) {  // utenfor akseptabel vinkel
									correction = leftfast;
								} else if ((headData < 359-3 && headData > 180)) {  // utenfor akseptabel vinkel
									correction = rightfast;
								} else {
									correction = none;
								}
								switch (correction) {
									case left:
										setSwing(-1);
										swing = straight;
										startTimer(25,t4);
										break;
									case right:
										setSwing(1);
										swing = straight;
										startTimer(25,t4);
										break;
									case leftfast:
										setSwing(-2);
										swing = straight;
										startTimer(25,t4);
										break;
									case rightfast:
										setSwing(2);
										swing = straight;
										startTimer(25,t4);
										break;
									case none:
										setSwing(0);
										break;
								}
							}
							break;
				}
				break;

			// sr
			case S:
				 // sving timeout
				 switch (swing) {
				 		case turn:
							// stopp sving, kjr et stykke, g til straight modus
							if (timeout(t4)) {
							 	setSwing(0);

								// kjr frem litt
								swing = straight;
								startTimer(700,t4);
							}
							break;
				 		case straight:
							// stopp sving, sett hastighet, g til navigate modus
							if (timeout(t4)) {
							 	setSwing(0);
								setSpeed(10);
								swing = navigate;
							}
							break;
						case navigate:

							// pent til hyre, snu mot vest
							if ((radarDist2 > 220 && radarDist1 < 180) || (posData.x < 400 && posData.x > 200 && posData.y < 215 && radarDist1 > 180)) {
								setSwing(2);
								swing = turn;
								startTimer(1150,t4);
								dir = W;

							// pent til venstre, snu mot st
							} else if (radarDist0 > 220) {
								setSwing(-2);
								swing = turn;
								startTimer(1150,t4);
								dir = E;

							// stengt forran, snu 180
							} else if (radarDist1 < 100) {
								setSpeed(0);
								setSwing(10);
								swing = straight;
								startTimer(920,t4);
								dir = N;

							// rett frem, korrigering
							} else {
								if ((radarDist0 < 190 && radarDist2 < 190 && radarDist1 > 200 && // vegg begge sider
									radarDist0 < radarDist2-18 && // venstre lavere enn hyre
									headData < 195 && headData > 165)) { // utenfor akseptabel vinkel

									correction = rightfast;
								} else if ((radarDist0 < 190 && radarDist2 < 190 && radarDist1 > 200 && // vegg begge sider
									radarDist2 < radarDist0-18 && // hyre lavere enn venstre
									headData > 165 && headData < 195)) { // utenfor akseptabel vinkel

									correction = leftfast;
								} else if (headData > 180+3) { // utenfor akseptabel vinkel
									correction = leftfast;
								} else if (headData < 180-3) { // utenfor akseptabel vinkel
									correction = rightfast;
								} else {
									correction = none;
								}
								switch (correction) {
									case left:
										setSwing(-1);
										swing = straight;
										startTimer(25,t4);
										break;
									case right:
										setSwing(1);
										swing = straight;
										startTimer(25,t4);
										break;
									case leftfast:
										setSwing(-2);
										swing = straight;
										startTimer(25,t4);
										break;
									case rightfast:
										setSwing(2);
										swing = straight;
										startTimer(25,t4);
										break;
									case none:
										setSwing(0);
										break;
								}

							}
							break;
					}
					break;

			// st
			case E:
				 // sving timeout
				 switch (swing) {
				 		case turn:
							// stopp sving, kjr et stykke, g til straight modus
							if (timeout(t4)) {
							 	setSwing(0);

								// kjr frem litt
								swing = straight;
								startTimer(500,t4);
							}
							break;
				 		case straight:
							// stopp sving, sett hastighet, g til navigate modus
							if (timeout(t4)) {
								setSpeed(10);
							 	setSwing(0);
								swing = navigate;
							}
							break;
						case navigate:
							// pent til hyre, snu mot sr
							if (radarDist2 > 220) {
								setSwing(-2);
								swing = turn;
								startTimer(1150,t4);
								dir = N;

							// pent til venstre, snu mot sr
							} else if (radarDist0 > 220) {
								setSwing(2);
								swing = turn;
								startTimer(1150,t4);
								dir = S;

							// stengt forran, snu
							} else if (radarDist1 < 100) {
								setSwing(10);
								setSpeed(0);
								swing = straight;
								startTimer(700,t4);
								dir = W;

							// rett frem, korrigering
							} else {
								if (headData < 90-1) { // utenfor akseptabel vinkel
									correction = right;
								} else if (headData > 90+1) { // utenfor akseptabel vinkel
									correction = left;
								} else {
									correction = none;
								}
								switch (correction) {
									case left:
										setSwing(-1);
										swing = straight;
										startTimer(25,t4);
										break;
									case right:
										setSwing(1);
										swing = straight;
										startTimer(25,t4);
										break;
									case none:
										setSwing(0);
										break;
									case leftfast:
										break;
									case rightfast:
										break;
								}

							}
							break;
					}
					break;

			// vest
			case W:
				 // sving timeout
				 switch (swing) {
				 		case turn:
							// stopp sving, kjr et stykke, g til straight modus
							if (timeout(t4)) {
							 	setSwing(0);

								// kjr frem litt
								swing = straight;
								startTimer(500,t4); 
							}
							break;
				 		case straight:
							// stopp sving, sett hastighet, g til navigate modus
							if (timeout(t4)) {
							 	setSwing(0);
								setSpeed(10);
								swing = navigate;
							}
							break;
						case navigate:
							// pent til venstre, snu mot sr
							if (radarDist0 > 220) {
								setSwing(-2);
								swing = turn;
								startTimer(1150,t4);
								dir = S;

							// pent til hyre, snu mot nord
							} else if (radarDist2 > 220) {
								setSwing(2);
								swing = turn;
								startTimer(1150,t4);
								dir = N;

							// stengt forran, snu
							} else if (radarDist1 < 100) {
								setSwing(-10);
								setSpeed(0);
								swing = straight;
								startTimer(700,t4);
								dir = E;

							// rett frem, korrigering
							} else {
								if (headData < 270-1) { // utenfor akseptabel vinkel
									correction = right;
								} else if (headData > 270+1) { // utenfor akseptabel vinkel
									correction = left;
								} else {
									correction = none;
								}
								switch (correction) {
									case left:
										setSwing(-1);
										swing = straight;
										startTimer(25,t4);
										break;
									case right:
										setSwing(1);
										swing = straight;
										startTimer(25,t4);
										break;
									case leftfast:
										break;
									case rightfast:
										break;
									case none:
										setSwing(0);
										break;
								}

							}
							break;
				}
				break;

			// g rett frem til ml
			case X:
				//setSwing(0);

				// stopp ved mlgang
				if (obstacleHit()) {
					setSpeed(-10);
					startTimer(500,t4);
					dir = Y;
					//setSpeed(0);
				}

				if (timeout(t4)) {
					startTimer(100+finalswing,t4);
					setSwing(10);
					finalswing += 100;
					dir = Y;
				}
				break;

			case Y:
				if (timeout(t4)) {
					setSpeed(0);
					startTimer(100+finalswing,t4);
					setSwing(-10);
					finalswing += 100;
					dir = X;
				}
				break;
		}	
		sendSignal(displayDir, &dir);

	}
}

int main(void)
{
	setSpeed(0);
	setSwing(0);

	//waitNms(5000);
	unsigned int x; 
	unsigned int n=5000; 
	while(n--){ 
	    x=500; // fudge factor 16mhz 
	    while(x--); 
	} 

	radar0 = newSignal(1, sizeof(int));
	radar1 = newSignal(1, sizeof(int));
	radar2 = newSignal(1, sizeof(int));
	pos = newSignal(1, sizeof(position_type));
	head = newSignal(1, sizeof(int));

	displayRadar0 = newSignal(1, sizeof(int));
	displayRadar1 = newSignal(1, sizeof(int));
	displayRadar2 = newSignal(1, sizeof(int));
	displayPos = newSignal(1, sizeof(position_type));
	displayHead = newSignal(1, sizeof(int));
	displayDir = newSignal(1, sizeof(int));

	while(1) {
		radar_process();
		control_process();
		display_process();
		data_process();
	}
	return 0;
}
